@@ -89,6 +89,7 @@ xmlNodePtr _msMetadataGetOnline(xmlNsPtr namespace, layerObj *layer, char *servi
   char *url = NULL;
   char buffer[32];
   char *epsg_str;
+  char *link_protocol;
 
   xmlNodePtr psNode = NULL;
   xmlNodePtr psORNode = NULL;
@@ -108,6 +109,7 @@ xmlNodePtr _msMetadataGetOnline(xmlNsPtr namespace, layerObj *layer, char *servi
     url = msStringConcatenate(url, msEncodeHTMLEntities("&crs="));
     msOWSGetEPSGProj(&(layer->projection), &(layer->metadata), "MFCSGO", MS_TRUE, &epsg_str);
     url = msStringConcatenate(url, msEncodeHTMLEntities(epsg_str));
+    link_protocol = "WWW:DOWNLOAD-1.0-http-get-map";
 
     status = msLayerGetExtent(layer, &rect);
 
@@ -127,12 +129,14 @@ xmlNodePtr _msMetadataGetOnline(xmlNsPtr namespace, layerObj *layer, char *servi
     }
   }
   else if (strcasecmp(service, "F") == 0) {
+    link_protocol = "WWW:DOWNLOAD-1.0-http--download";
     url = msStringConcatenate(url, msEncodeHTMLEntities("service=WFS&version=1.1.0&request=GetFeature&typename="));
     url = msStringConcatenate(url, msEncodeHTMLEntities(layer->name));
     url = msStringConcatenate(url, msEncodeHTMLEntities("&outputformat="));
     url = msStringConcatenate(url, msEncodeHTMLEntities(format));
   }
   else if (strcasecmp(service, "C") == 0) {
+    link_protocol = "WWW:DOWNLOAD-1.0-http--download";
     url = msStringConcatenate(url, msEncodeHTMLEntities("service=WCS&version=2.0.1&request=GetCoverage&coverageid="));
     url = msStringConcatenate(url, msEncodeHTMLEntities(layer->name));
     url = msStringConcatenate(url, msEncodeHTMLEntities("&format="));
@@ -141,7 +145,7 @@ xmlNodePtr _msMetadataGetOnline(xmlNsPtr namespace, layerObj *layer, char *servi
 
   xmlAddChild(psORNode, _msMetadataGetURL(namespace, "linkage", url));
 
-  xmlAddChild(psORNode, _msMetadataGetCharacterString(namespace, "protocol", "WWW:DOWNLOAD-1.0-http--download"));
+  xmlAddChild(psORNode, _msMetadataGetCharacterString(namespace, "protocol", link_protocol));
   xmlAddChild(psORNode, _msMetadataGetCharacterString(namespace, "name", layer->name));
 
   xmlAddChild(psORNode, _msMetadataGetCharacterString(namespace, "description", desc));
@@ -450,12 +454,12 @@ xmlNodePtr _msMetadataGetContact(xmlNsPtr namespace, char *contact_element, mapO
   if (value)
     xmlAddChild(psCIAddressNode, _msMetadataGetCharacterString(namespace, "electronicMailAddress", value));
 
-  psORNode = xmlNewChild(psCINode2, namespace, BAD_CAST "onlineResource", NULL);
-  psORNode2 = xmlNewChild(psORNode, namespace, BAD_CAST "CI_OnlineResource", NULL);
-
   value = (char *)msOWSLookupMetadata(&(map->web.metadata), "MCFO", "onlineresource");
-  if (value)
+  if (value) {
+    psORNode = xmlNewChild(psCINode2, namespace, BAD_CAST "onlineResource", NULL);
+    psORNode2 = xmlNewChild(psORNode, namespace, BAD_CAST "CI_OnlineResource", NULL);
     xmlAddChild(psORNode2, _msMetadataGetURL(namespace, "linkage", value));
+  }
 
   xmlAddChild(psCNode, _msMetadataGetCodeList(namespace, "role", "CI_RoleCode", "pointOfContact"));
 
@@ -604,9 +608,7 @@ xmlNodePtr _msMetadataGetDistributionInfo(xmlNsPtr namespace, mapObj *map, layer
 
   /* WMS */
   xmlAddChild(psDTONode, _msMetadataGetOnline(namespace, layer, "M", "image/png", "PNG Format", url));
-
   xmlAddChild(psDTONode, _msMetadataGetOnline(namespace, layer, "M", "image/jpeg", "JPEG Format", url));
-  xmlAddChild(psDTONode, _msMetadataGetOnline(namespace, layer, "M", "image/gif", "GIF Format", url));
 
   /* WCS */
   if (layer->type == MS_LAYER_RASTER) {
@@ -892,10 +894,10 @@ void msMetadataSetGetMetadataURL(layerObj *lp, const char *url)
   pszMetadataURL = msStringConcatenate(pszMetadataURL, lp->name);
 
   msInsertHashTable(&(lp->metadata), "ows_metadataurl_href", pszMetadataURL);
-  msInsertHashTable(&(lp->metadata), "ows_metadataurl_type", "ISOTC211/19115");
+  msInsertHashTable(&(lp->metadata), "ows_metadataurl_type", "TC211");
   msInsertHashTable(&(lp->metadata), "ows_metadataurl_format", "text/xml");
   msInsertHashTable(&(lp->metadata), "ows_metadatalink_href", pszMetadataURL);
-  msInsertHashTable(&(lp->metadata), "ows_metadatalink_type", "ISOTC211/19115");
+  msInsertHashTable(&(lp->metadata), "ows_metadatalink_type", "TC211");
   msInsertHashTable(&(lp->metadata), "ows_metadatalink_format", "text/xml");
   msFree(pszMetadataURL);
 }
