@@ -212,7 +212,7 @@ typedef union YYSTYPE
   shapeObj *shpval;
 }
 /* Line 193 of yacc.c.  */
-#line 216 "/Users/tbonfort/dev/mapserver/mapparser.c"
+#line 216 "/Users/sdlime/mapserver/sdlime/mapserver/mapparser.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -225,7 +225,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 229 "/Users/tbonfort/dev/mapserver/mapparser.c"
+#line 229 "/Users/sdlime/mapserver/sdlime/mapserver/mapparser.c"
 
 #ifdef short
 # undef short
@@ -1700,9 +1700,9 @@ yyreduce:
       break;
     case(MS_PARSE_TYPE_STRING):
       if((yyvsp[(1) - (1)].intval)) 
-        p->result.strval = strdup("true");
+        p->result.strval = msStrdup("true");
       else
-        p->result.strval = strdup("false");
+        p->result.strval = msStrdup("false");
       break;
     }
   }
@@ -1737,7 +1737,7 @@ yyreduce:
         p->result.intval = MS_FALSE;
       break;
     case(MS_PARSE_TYPE_STRING):
-      p->result.strval = (yyvsp[(1) - (1)].strval); // strdup($1);
+      p->result.strval = (yyvsp[(1) - (1)].strval); // msStrdup($1);
       break;
     }
   }
@@ -2755,7 +2755,7 @@ yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 2759 "/Users/tbonfort/dev/mapserver/mapparser.c"
+#line 2759 "/Users/sdlime/mapserver/sdlime/mapserver/mapparser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -3002,7 +3002,7 @@ int yylex(YYSTYPE *lvalp, parseObj *p)
   case MS_TOKEN_LITERAL_STRING:
     // printf("token value = %s\n", p->expr->curtoken->tokenval.strval); 
     token = STRING;
-    (*lvalp).strval = strdup(p->expr->curtoken->tokenval.strval);    
+    (*lvalp).strval = msStrdup(p->expr->curtoken->tokenval.strval);    
     break;
   case MS_TOKEN_LITERAL_TIME:
     token = TIME;
@@ -3019,6 +3019,8 @@ int yylex(YYSTYPE *lvalp, parseObj *p)
   case MS_TOKEN_COMPARISON_RE: token = RE; break;
   case MS_TOKEN_COMPARISON_IRE: token = IRE; break;
 
+  case MS_TOKEN_COMPARISON_IN: token = IN; break;
+
   case MS_TOKEN_COMPARISON_INTERSECTS: token = INTERSECTS; break;
   case MS_TOKEN_COMPARISON_DISJOINT: token = DISJOINT; break;
   case MS_TOKEN_COMPARISON_TOUCHES: token = TOUCHES; break;
@@ -3041,7 +3043,7 @@ int yylex(YYSTYPE *lvalp, parseObj *p)
     break;
   case MS_TOKEN_BINDING_STRING:
     token = STRING;
-    (*lvalp).strval = strdup(p->shape->values[p->expr->curtoken->tokenval.bindval.index]);
+    (*lvalp).strval = msStrdup(p->shape->values[p->expr->curtoken->tokenval.bindval.index]);
     break;
   case MS_TOKEN_BINDING_SHAPE:
     token = SHAPE;
