@@ -28,6 +28,22 @@
  ****************************************************************************/
 
 #include "mapserver.h"
+#include "mapows.h"
+
+/* This object is used by the various mapQueryXXXXX() functions. It stores
+ * the total amount of shapes and their RAM footprint, when they are cached
+ * in the resultCacheObj* of layers. This is the total number accross all queried
+ * layers. However this is isn't persistant accross several calls to
+ * mapQueryXXXXX(), if the resultCacheObj* objects weren't cleaned up. This
+ * is not needed in the context of WFS, for which this is used for now.
+ */
+typedef struct
+{
+    int cachedShapeCountWarningEmitted;
+    int cachedShapeCount;
+    int cachedShapeRAMWarningEmitted;
+    int cachedShapeRAM;
+} queryCacheObj;
 
 int msInitQuery(queryObj *query) {
   if(!query) return MS_FAILURE;
@@ -55,6 +71,10 @@ int msInitQuery(queryObj *query) {
   
   query->filteritem = NULL;
   msInitExpression(&query->filter);
+  
+  query->cache_shapes = MS_FALSE;
+  query->max_cached_shape_count = 0;
+  query->max_cached_shape_ram_amount = 0;
 
   return MS_SUCCESS;
 }
@@ -132,9 +152,60 @@ int msIsLayerQueryable(layerObj *lp)
   return MS_FALSE;
 }
 
-static int addResult(resultCacheObj *cache, shapeObj *shape)
+static void initQueryCache(queryCacheObj* queryCache)
+{
+    queryCache->cachedShapeCountWarningEmitted = MS_FALSE;
+    queryCache->cachedShapeCount = 0;
+    queryCache->cachedShapeRAMWarningEmitted = MS_FALSE;
+    queryCache->cachedShapeRAM = 0;
+}
+
+/** Check whether we should store the shape in resultCacheObj* given the
+ * limits allowed in map->query.max_cached_shape_count and
+ * map->query.max_cached_shape_ram_amount.
+ */
+static int canCacheShape(mapObj* map, queryCacheObj *queryCache,
+                         shapeObj* shape, int shape_ram_size)
+{
+  if( !map->query.cache_shapes )
+      return MS_FALSE;
+  if( queryCache->cachedShapeCountWarningEmitted ||
+      (map->query.max_cached_shape_count > 0 &&
+      queryCache->cachedShapeCount >= map->query.max_cached_shape_count) )
+  {
+      if( !queryCache->cachedShapeCountWarningEmitted )
+      {
+          queryCache->cachedShapeCountWarningEmitted = MS_TRUE;
+          msDebug("map->query.max_cached_shape_count = %d reached. "
+                  "Next features will not be cached.\n",
+                  map->query.max_cached_shape_count);
+      }
+      return MS_FALSE;
+  }
+  if( queryCache->cachedShapeRAMWarningEmitted ||
+      (map->query.max_cached_shape_ram_amount > 0 &&
+       queryCache->cachedShapeRAM + shape_ram_size > map->query.max_cached_shape_ram_amount) )
+  {
+      if( !queryCache->cachedShapeRAMWarningEmitted )
+      {
+          queryCache->cachedShapeRAMWarningEmitted = MS_TRUE;
+          msDebug("map->query.max_cached_shape_ram_amount = %d reached after %d cached features. "
+                  "Next features will not be cached.\n",
+                  map->query.max_cached_shape_ram_amount,
+                  queryCache->cachedShapeCount);
+      }
+      return MS_FALSE;
+  }
+  return MS_TRUE;
+}
+
+static int addResult(mapObj* map, resultCacheObj *cache,
+                     queryCacheObj* queryCache, shapeObj *shape)
 {
   int i;
+  int shape_ram_size = (map->query.max_cached_shape_ram_amount > 0) ? 
+                                            msGetShapeRAMSize( shape ) : 0;
+  int store_shape = canCacheShape (map, queryCache, shape, shape_ram_size);
 
   if(cache->numresults == cache->cachesize) { /* just add it to the end */
     if(cache->cachesize == 0)
@@ -154,6 +225,16 @@ static int addResult(resultCacheObj *cache, shapeObj *shape)
   cache->results[i].tileindex = shape->tileindex;
   cache->results[i].shapeindex = shape->index;
   cache->results[i].resultindex = shape->resultindex;
+  if( store_shape )
+  {
+      cache->results[i].shape = (shapeObj*)msSmallMalloc(sizeof(shapeObj));
+      msInitShape(cache->results[i].shape);
+      msCopyShape(shape, cache->results[i].shape);
+      queryCache->cachedShapeCount ++;
+      queryCache->cachedShapeRAM += shape_ram_size;
+  }
+  else
+      cache->results[i].shape = NULL;
   cache->numresults++;
 
   cache->previousBounds = cache->bounds;
@@ -470,10 +551,13 @@ int msQueryByIndex(mapObj *map)
   int status;
 
   resultObj record;
+  queryCacheObj queryCache;
 
   shapeObj shape;
   double minfeaturesize = -1;
 
+  initQueryCache(&queryCache);
+
   if(map->query.type != MS_QUERY_BY_INDEX) {
     msSetError(MS_QUERYERR, "The query is not properly defined.", "msQueryByIndex()");
     return(MS_FAILURE);
@@ -493,7 +577,7 @@ int msQueryByIndex(mapObj *map)
 
   if(map->query.clear_resultcache) {
     if(lp->resultcache) {
-      if(lp->resultcache->results) free(lp->resultcache->results);
+      cleanupResultCache(lp->resultcache);
       free(lp->resultcache);
       lp->resultcache = NULL;
     }
@@ -567,7 +651,7 @@ int msQueryByIndex(mapObj *map)
     return(MS_FAILURE);
   }
   
-  addResult(lp->resultcache, &shape);
+  addResult(map, lp->resultcache, &queryCache, &shape);
 
   msFreeShape(&shape);
   /* msLayerClose(lp); */
@@ -575,30 +659,11 @@ int msQueryByIndex(mapObj *map)
   return(MS_SUCCESS);
 }
 
-void msRestoreOldFilter(layerObj *lp, int old_filtertype, char *old_filteritem, char *old_filterstring)
-{
-  msFreeExpression(&(lp->filter));
-  if(lp->filteritem) {
-    free(lp->filteritem);
-    lp->filteritem = NULL;
-    lp->filteritemindex = -1;
-  }
-
-  /* restore any previously defined filter */
-  if(old_filterstring) {
-    lp->filter.type = old_filtertype;
-    lp->filter.string = old_filterstring;
-    if(old_filteritem) {
-      lp->filteritem = old_filteritem;
-    }
-  }
-}
-
 static char *filterTranslateToLogical(expressionObj *filter, char *filteritem) {
   char *string = NULL;
 
   if(filter->type == MS_STRING && filteritem) {
-    string = strdup("'[");
+    string = msStrdup("'[");
     string = msStringConcatenate(string, filteritem);
     string = msStringConcatenate(string, "]'");
     if(filter->flags & MS_EXP_INSENSITIVE)
@@ -608,7 +673,7 @@ static char *filterTranslateToLogical(expressionObj *filter, char *filteritem) {
     string = msStringConcatenate(string, filter->string);
     string = msStringConcatenate(string, "'");
   } else if(filter->type == MS_REGEX && filteritem) {
-    string = strdup("'[");
+    string = msStrdup("'[");
     string = msStringConcatenate(string, filteritem);
     string = msStringConcatenate(string, "]'");
     if(filter->flags & MS_EXP_INSENSITIVE)
@@ -644,7 +709,7 @@ static expressionObj mergeFilters(expressionObj *filter1, char *filteritem1, exp
     return filter; /* should only happen if the filter was a native filter */
   }
 
-  filter.string = strdup(tmpstr1);
+  filter.string = msStrdup(tmpstr1);
   filter.string = msStringConcatenate(filter.string, " AND ");
   filter.string = msStringConcatenate(filter.string, tmpstr2);
 
@@ -678,6 +743,9 @@ int msQueryByFilter(mapObj *map)
   int nclasses = 0;
   int *classgroup = NULL;
   double minfeaturesize = -1;
+  queryCacheObj queryCache;
+
+  initQueryCache(&queryCache);
 
   if(map->query.type != MS_QUERY_BY_FILTER) {
     msSetError(MS_QUERYERR, "The query is not properly defined.", "msQueryByFilter()");
@@ -766,6 +834,65 @@ int msQueryByFilter(mapObj *map)
 
     search_rect = map->query.rect;
 
+    /* If only result count is needed, we can use msLayerGetShapeCount() */
+    /* that has optimizations to avoid retrieving individual features */
+    if( map->query.only_cache_result_count &&
+        lp->template != NULL && /* always TRUE for WFS case */
+        lp->minfeaturesize <= 0 )
+    {
+      int bUseLayerSRS = MS_FALSE;
+      int numFeatures = -1;
+
+#if defined(USE_PROJ) && (defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR) || defined(USE_WMS_LYR) || defined(USE_WFS_LYR))
+      /* Optimization to detect the case where a WFS query uses in fact the */
+      /* whole layer extent, but expressed in a map SRS different from the layer SRS */
+      /* In the case, we can directly request against the layer extent in its native SRS */
+      if( lp->project &&
+          memcmp( &search_rect, &invalid_rect, sizeof(search_rect) ) != 0 &&
+          msProjectionsDiffer(&(lp->projection), &(map->projection)) )
+      {
+        rectObj layerExtent;
+        if ( msOWSGetLayerExtent(map, lp, "FO", &layerExtent) == MS_SUCCESS)
+        {
+            rectObj ext = layerExtent;
+            ext.minx -= 1e-5;
+            ext.miny -= 1e-5;
+            ext.maxx += 1e-5;
+            ext.maxy += 1e-5;
+            msProjectRect(&(lp->projection), &(map->projection), &ext);
+            if( fabs(ext.minx - search_rect.minx) <= 2e-5 &&
+                fabs(ext.miny - search_rect.miny) <= 2e-5 &&
+                fabs(ext.maxx - search_rect.maxx) <= 2e-5 &&
+                fabs(ext.maxy - search_rect.maxy) <= 2e-5 )
+            {
+              bUseLayerSRS = MS_TRUE;
+              numFeatures = msLayerGetShapeCount(lp, layerExtent, &(lp->projection));
+            }
+        }
+      }
+#endif
+
+      if( !bUseLayerSRS )
+          numFeatures = msLayerGetShapeCount(lp, search_rect, &(map->projection));
+      if( numFeatures >= 0 )
+      {
+        lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
+        MS_CHECK_ALLOC(lp->resultcache, sizeof(resultCacheObj), MS_FAILURE);
+        initResultCache( lp->resultcache);
+        lp->resultcache->numresults = numFeatures;
+        if (!msLayerGetPaging(lp) && map->query.startindex > 1) {
+           lp->resultcache->numresults -= (map->query.startindex-1);
+        }
+
+        lp->filteritem = old_filteritem; /* point back to original value */
+        msCopyExpression(&lp->filter, &old_filter); /* restore old filter */
+        msFreeExpression(&old_filter);
+
+        continue;
+      }
+      // Fallback in case of error (should not happen normally)
+    }
+
 #ifdef USE_PROJ
     lp->project = msProjectionsDiffer(&(lp->projection), &(map->projection));
     if(lp->project && memcmp( &search_rect, &invalid_rect, sizeof(search_rect) ) != 0 )
@@ -827,7 +954,7 @@ int msQueryByFilter(mapObj *map)
       if( map->query.only_cache_result_count )
         lp->resultcache->numresults ++;
       else
-        addResult(lp->resultcache, &shape);
+        addResult(map, lp->resultcache, &queryCache, &shape);
       msFreeShape(&shape);
 
       if(map->query.mode == MS_QUERY_SINGLE) { /* no need to look any further */
@@ -888,6 +1015,9 @@ int msQueryByRect(mapObj *map)
   int nclasses = 0;
   int *classgroup = NULL;
   double minfeaturesize = -1;
+  queryCacheObj queryCache;
+
+  initQueryCache(&queryCache);
 
   if(map->query.type != MS_QUERY_BY_RECT) {
     msSetError(MS_QUERYERR, "The query is not properly defined.", "msQueryByRect()");
@@ -981,6 +1111,61 @@ int msQueryByRect(mapObj *map)
       return(MS_FAILURE);
     }
 
+    /* If only result count is needed, we can use msLayerGetShapeCount() */
+    /* that has optimizations to avoid retrieving individual features */
+    if( map->query.only_cache_result_count &&
+        lp->template != NULL && /* always TRUE for WFS case */
+        lp->minfeaturesize <= 0 )
+    {
+      int bUseLayerSRS = MS_FALSE;
+      int numFeatures = -1;
+
+#if defined(USE_PROJ) && (defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR) || defined(USE_WMS_LYR) || defined(USE_WFS_LYR))
+      /* Optimization to detect the case where a WFS query uses in fact the */
+      /* whole layer extent, but expressed in a map SRS different from the layer SRS */
+      /* In the case, we can directly request against the layer extent in its native SRS */
+      if( lp->project &&
+          memcmp( &searchrect, &invalid_rect, sizeof(searchrect) ) != 0 &&
+          msProjectionsDiffer(&(lp->projection), &(map->projection)) )
+      {
+        rectObj layerExtent;
+        if ( msOWSGetLayerExtent(map, lp, "FO", &layerExtent) == MS_SUCCESS)
+        {
+            rectObj ext = layerExtent;
+            ext.minx -= 1e-5;
+            ext.miny -= 1e-5;
+            ext.maxx += 1e-5;
+            ext.maxy += 1e-5;
+            msProjectRect(&(lp->projection), &(map->projection), &ext);
+            if( fabs(ext.minx - searchrect.minx) <= 2e-5 &&
+                fabs(ext.miny - searchrect.miny) <= 2e-5 &&
+                fabs(ext.maxx - searchrect.maxx) <= 2e-5 &&
+                fabs(ext.maxy - searchrect.maxy) <= 2e-5 )
+            {
+              bUseLayerSRS = MS_TRUE;
+              numFeatures = msLayerGetShapeCount(lp, layerExtent, &(lp->projection));
+            }
+        }
+      }
+#endif
+
+      if( !bUseLayerSRS )
+          numFeatures = msLayerGetShapeCount(lp, searchrect, &(map->projection));
+      if( numFeatures >= 0 )
+      {
+        lp->resultcache = (resultCacheObj *)malloc(sizeof(resultCacheObj)); /* allocate and initialize the result cache */
+        MS_CHECK_ALLOC(lp->resultcache, sizeof(resultCacheObj), MS_FAILURE);
+        initResultCache( lp->resultcache);
+        lp->resultcache->numresults = numFeatures;
+        if (!paging && map->query.startindex > 1) {
+           lp->resultcache->numresults -= (map->query.startindex-1);
+        }
+        msFreeShape(&searchshape);
+        continue;
+      }
+      // Fallback in case of error (should not happen normally)
+    }
+
 #ifdef USE_PROJ
     lp->project = msProjectionsDiffer(&(lp->projection), &(map->projection));
     if(lp->project &&
@@ -1066,7 +1251,7 @@ int msQueryByRect(mapObj *map)
         if( map->query.only_cache_result_count )
             lp->resultcache->numresults ++;
         else
-            addResult(lp->resultcache, &shape);
+            addResult(map, lp->resultcache, &queryCache, &shape);
         --map->query.maxfeatures;
       }
       msFreeShape(&shape);
@@ -1128,6 +1313,10 @@ int msQueryByFeatures(mapObj *map)
   int *classgroup = NULL;
   double minfeaturesize = -1;
 
+  queryCacheObj queryCache;
+
+  initQueryCache(&queryCache);
+
   if(map->debug) msDebug("in msQueryByFeatures()\n");
 
   /* is the selection layer valid and has it been queried */
@@ -1384,7 +1573,7 @@ int msQueryByFeatures(mapObj *map)
             msFreeShape(&shape);
             continue;
           }
-          addResult(lp->resultcache, &shape);
+          addResult(map, lp->resultcache, &queryCache, &shape);
         }
         msFreeShape(&shape);
 
@@ -1448,6 +1637,10 @@ int msQueryByPoint(mapObj *map)
   int *classgroup = NULL;
   double minfeaturesize = -1;
 
+  queryCacheObj queryCache;
+
+  initQueryCache(&queryCache);
+
   if(map->query.type != MS_QUERY_BY_POINT) {
     msSetError(MS_QUERYERR, "The query is not properly defined.", "msQueryByPoint()");
     return(MS_FAILURE);
@@ -1603,11 +1796,12 @@ int msQueryByPoint(mapObj *map)
         }
 
         if(map->query.mode == MS_QUERY_SINGLE) {
-          lp->resultcache->numresults = 0;
-          addResult(lp->resultcache, &shape);
+          cleanupResultCache(lp->resultcache);
+          initQueryCache(&queryCache);
+          addResult(map, lp->resultcache, &queryCache, &shape);
           t = d; /* next one must be closer */
         } else {
-          addResult(lp->resultcache, &shape);
+          addResult(map, lp->resultcache, &queryCache, &shape);
         }
       }
 
@@ -1658,6 +1852,9 @@ int msQueryByShape(mapObj *map)
   int nclasses = 0;
   int *classgroup = NULL;
   double minfeaturesize = -1;
+  queryCacheObj queryCache;
+
+  initQueryCache(&queryCache);
 
   if(map->query.type != MS_QUERY_BY_SHAPE) {
     msSetError(MS_QUERYERR, "The query is not properly defined.", "msQueryByShape()");
@@ -1891,7 +2088,7 @@ int msQueryByShape(mapObj *map)
           msFreeShape(&shape);
           continue;
         }
-        addResult(lp->resultcache, &shape);
+        addResult(map, lp->resultcache, &queryCache, &shape);
       }
       msFreeShape(&shape);
 
