@@ -113,7 +113,7 @@ PHP_METHOD(pointObj, __construct)
 PHP_METHOD(pointObj, __get)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *zobj = getThis();
   php_point_object *php_point;
 
@@ -141,7 +141,7 @@ PHP_METHOD(pointObj, __get)
 PHP_METHOD(pointObj, __set)
 {
   char *property;
-  long property_len;
+  long property_len = 0;
   zval *value;
   zval *zobj = getThis();
   php_point_object *php_point;
@@ -355,7 +355,7 @@ PHP_METHOD(pointObj, draw)
   zval *zobj =  getThis();
   zval *zmap, *zlayer, *zimage;
   char *text = NULL;
-  long text_len;
+  long text_len = 0;
   long classIndex;
   int status = MS_FAILURE;
   php_point_object *php_point;
