@@ -36,8 +36,7 @@ import os, sys
 import unittest
 
 # the testing module helps us import the pre-installed mapscript
-from testing import mapscript
-from testing import MapTestCase
+from .testing import MapTestCase, mapscript
 
 # ===========================================================================
 # Test begins now
