@@ -1,4 +1,4 @@
-# $Id: refcount.py 7205 2007-12-22 09:13:59Z unicoletti $
+# $Id$
 #
 # Project:  MapServer
 # Purpose:  xUnit style Python mapscript tests of Map
@@ -36,7 +36,7 @@ import os, sys
 import unittest
 
 # the testing module helps us import the pre-installed mapscript
-from testing import mapscript, MapTestCase, TESTMAPFILE
+from .testing import mapscript, MapTestCase, TESTMAPFILE
 
 # ===========================================================================
 # Test begins now
