@@ -33,7 +33,7 @@
 #
 # All test case modules should import mapscript from testing
 #
-#     from testing import mapscript
+#     from .testing import mapscript
 #
 # ===========================================================================
 
@@ -43,7 +43,8 @@ import distutils.util
 import unittest
 
 # define the path to mapserver test data
-TESTS_PATH = '../../../../tests'
+test_path = os.path.abspath(os.path.join(__file__ ,"../../../../.."))
+TESTS_PATH = os.path.join(test_path, "tests")
 
 TESTMAPFILE = os.path.join(TESTS_PATH, 'test.map')
 XMARKS_IMAGE = os.path.join(TESTS_PATH, 'xmarks.png')
@@ -63,15 +64,7 @@ import mapscript
 # ==========================================================================
 # Base testing classes
 
-class MapscriptTestCase(unittest.TestCase):
-
-    def assertAlmostEqual(self, first, second, places=7):
-        """Copied from unittest for use with Python 2.1 or 2.2"""
-        if round(second-first, places) != 0:
-            raise AssertionError, \
-                '%s != %s within %s places' % (`first`, `second`, `places`)
-        
-class MapPrimitivesTestCase(MapscriptTestCase):
+class MapPrimitivesTestCase(unittest.TestCase):
     """Base class for testing primitives (points, lines, shapes)
     in stand-alone mode"""
 
@@ -164,4 +157,3 @@ class MapZoomTestCase(MapPrimitivesTestCase):
 class ShapeObjTestCase(MapPrimitivesTestCase):
     """Base class for shapeObj tests"""
     pass
-
