@@ -347,6 +347,7 @@ static int msWCSGetCapabilities11_CoverageSummary(
                            format_list, ',' );
 
   msFree( format_list );
+  msWCSFreeCoverageMetadata(&cm);
 
   /* -------------------------------------------------------------------- */
   /*      Identifier (layer name)                                         */
@@ -879,6 +880,7 @@ msWCSDescribeCoverage_CoverageDescription11(
 
     msFree( format_list );
   }
+  msWCSFreeCoverageMetadata(&cm);
 
   return MS_SUCCESS;
 }
@@ -1233,6 +1235,7 @@ int  msWCSReturnCoverage11( wcsParamsObj *params, mapObj *map,
   /*      output a single "stock" filename.                               */
   /* -------------------------------------------------------------------- */
   if( filename == NULL ) {
+    msOutputFormatResolveFromImage( map, image );
     msIO_fprintf(
       stdout,
       "    <ows:Reference xlink:href=\"cid:coverage/wcs.%s\"/>\n"
